# 本サンプルコードについて

ここの内容は、技術評論社刊「WEB+DB PRESS Vol.124」の連載「Perl Hackers Hub」第69回「表形式データを操るPerl製UNIXシェル型コマンド群」（下野寿之著）で
紹介した各コマンドについて、一通り実行することで使い方が理解出来る様に、利用例を紹介したものです。
次に記載した **サンプルコード** は元の記事本文に書いてあるコードと同じものです。

下記のコマンドを実行するために、下記に現れるデータファイルとして`TRCOpenBibData_20210710.zip`の用意はしてあり、読者が利用可能後としています。しかし、`foo.tsv`と`twitter.tsv`は用意してありません。


# サンプルコード

## インストール

```
# 8個のコマンドを逐次インストール
$ cpanm App::csv2tsv
$ cpanm App::csel
$ cpanm App::expandtab
$ cpanm App::crosstable
$ cpanm App::colsummary
$ cpanm App::venn
$ cpanm App::digitdemog
$ cpanm App::colgrep

# 8個のコマンドを一括インストール
$ cpanm App::Bin4TSV::8

# plenvを使っている場合は、各コマンドを利用可能とするため以下を実行
$ plenv rehash
```

## csv2tsv

```
# CSV形式をTSV形式に変換。リダイレクション（<）の使用が望ましい
$ csv2tsv < foo.csv > foo.tsv
```
※ `foo.csv`は任意のCSV形式ファイルである。

## csel 

```
$ perl -E'say join"\t",1..12' | tee c12.tsv
1   2   3   4   5   6   7   8   9   10  11  12

# -pで指定列のみ抽出。「..」は範囲指定。AWKより簡単
$ csel -p3..7 c12.tsv
3   4   5   6   7

# -dで指定列を出力抑制。「,」は連結指定。AWKだと難しい
$ csel -d3..7,10,12 c12.tsv
1   2   8   9   11

# -hで指定列を先頭（左端）に移動。「12..10」は逆順指定（12,11,10）
$ csel -h12..10 c12.tsv
12  11  10  1   2   3   4   5   6   7   8   9

# -tで指定列を末尾（右端）に移動。レイアウトを崩す列の移動に実用的
$ csel -t9,6 c12.tsv
1   2   3   4   5   7   8   10  11  12  9   6
```

## expandtab

```
# -sで与えた閾値より幅広な文字列があると、その行の残りの列は狭くなる
$ expandtab -i: -s15 /etc/passwd
```

## crosstable

```
# 各行「年-月-日\t時:分:秒\t発言内容」の:以下をPerlで除去)
$ crosstable <( perl -pe's/:.*//' twitter.tsv )
```
※ twitter.tsvは提供されない。

## colsummary
```
# ダウンロードしたZIPファイルの解凍。次で使うT*0710.txtを生成
$ unzip TRCOpenBibData_20210710.zip

# 出力表の視認性のためにexpandtabを使用
$ colsummary -g3 -v9 -j -m0 T*0710.txt | expandtab -s30
```

### 結果

```
1,494 line(s) read; 0.092371 seconds (colsummary)
列番 異なる値 値の範囲                       最頻値                         頻度(重複)                   桁数
1    1485     |978-4-01-~978-4-991           |978-4-324|978-4-434           7|3|2|1(1482)                0|17
2    1489     #や【フラット】が~龍馬と西郷   お稲荷JKたまもち|THE STONE|THE IDOLM 2(5)|1(1484)           1~74
3    567      |#たっぷり濡らす#~黒糸1色で描く美し |大学入試|高校入試        889|8|5(2)~3(3)|2(12)|1(546) 0|2~72
4    1128     |AYA 著~龍牙翔 漫画            |協同教育研究会 編|日本医歯薬研修協会 231|41|8~3(8)|2(46)|1(1065) 0|3~39
5    352      |DIAMONDハー~黒輪篤嗣 訳       |荒山正彦 監修|わだことみ 監修・ 1116|4|3(6)|2(12)|1(332)   0|3~34
6    36       |2022年度入試用~第6版          |改訂版|復刻                   1383|23|14~3(4)|2(7)|1(17)   0|2~12
7    497      |ASK~黎明書房                  KADOKAWA|講談社|協同出版       70|66|41~3(35)|2(64)|1(293)  0|2~31
8    70       |JRC~風媒社                    |星雲社|幻冬舎                 1340|28|12~3(3)|2(6)|1(50)   0|3~15
9    8        2021~2021.8|c2021              2021.7|2021.6|2021.8           1168|187|79~16|5|3(2)        4~6
10   444      |10, 169p~p942〜1064           |1冊(ページ付なし|1冊          134|52|43~3(60)|2(77)|1(186) 0|2~14
11   41       |13×13cm~9.7×12cm              21cm|19cm|26cm                 281|260|254~4|2(7)|1(14)     0|4~8
12   7        |CD-ROM(1枚~録音ディスク(3枚   |録音ディスク(1枚|CD-ROM(1枚   1473|12|4|2|1(3)             0|15~19
13   526      |&arche NO~鹿児島県の教員採用  |週刊少年マガジン|ガンガンコミックス 753|21|14~3(14)|2(31)|1(455) 0|2~34
14   49       |<2 in 1>コ~麗人uno!           |GUSH COMI|光文社時代小説文庫  1425|6|5~3(2)|2(5)|1(38)     0|4~33
15   7        |Rouge~隠密船頭 7              |Rouge|隠密船頭 7              1486|3|1(5)                  0|5~33
16   165      |1925(大正14~饗宴              |未知なる鉱山と幻の|最終通貨の攻防 1330|1(164)              0|2~67
17   206      ¥1000~¥9900                    ¥1600|¥1500|¥1800              96|71|69~3(18)|2(27)|1(87)   4~6
18   5        |2巻セット¥270~全3巻セット¥23  |4巻セット¥120|全3巻セット¥23  1482|4|3(2)|2                0|10~12
```

## venn

```
# 自作シェル関数yを作成。引数を文字ごとにばらして改行を挿入
$ function y(){ echo -n $* | perl -pe's/./$&\n/g' }

# cardi.はcardinalityの略。minstrとmaxstrは辞書順の先頭と末尾
$ venn <(y ABCCC) <(y AABDEFG) <(y ABBDEFG) <(y DEEEFG)
```

### 結果

```
cardi.  file1   file2   file3   file4   minstr  maxstr
2.      2       3       3       0       'A'     'B'
1.      3       0       0       0       'C'
4.      0       4       4       6       'D'     'G'
```

## digitdemog
```
# 各文字が各行先頭から何文字目に何度出現したか。-.で追加のヒント
$ digitdemog -. <( csel -p11 T*0710.txt ) | expandtab
```

### 結果

```
    1   2   3     4     5     6   7   8   9
'.' 0   1.  0     0     0     0   0   0   0
'0' 0   132 0     0     7     0   0   0   0
'1' 504 308 0     7     4     0   0   0   0
'2' 753 70  0     22    3     1.  0   0   0
'3' 102 13  0     7     3     0   0   0   0
'4' 0   10  0     0     0     0   0   0   0
'5' 0   96  0     0     2     0   0   0   0
'6' 0   312 0     0     11    0   0   0   0
'7' 0   31  1.    0     3     0   0   0   0
'8' 0   101 0     0     1     0   0   0   0
'9' 1.  286 0     0     3     0   0   0   0
'c' 0   0   1323. 0     0     36. 1.  0   0
'm' 0   0   0     1323. 0     0   36. 1.  0
'×' 0   0   36.   1.    0     0   0   0   0
end 134 0   0     0     1323. 0   0   36. 1.
※各値の末尾に仮想的な文字endを伴うと見なしている
```

## colgrep 

```
# 1列目のISBNコードが空の図書をすべて検出。cselでタイトルなどのみ表示
$ colgrep -c1 -e'^$' T*0710.txt | csel -p11,17,2
```

### 結果

```
match:7 unmatch:1487 total:1494 (colgrep)
30cm    ¥1800   東京2020オリンピック公式ガイドブック
30cm    ¥1800   東京2020パラリンピック公式ガイドブック
21cm    ¥545    現代俳句 令和3年7月号
26cm    ¥8100   法令全書 令和3年5月号
30cm    ¥3000   賃金事情調査 令和2年
30cm    ¥4500   水産油脂統計年鑑 2020年
26cm    ¥1000   法と民主主義 NO.559(2021-6)
```

## colgrepの結果を比較する(1)

```
# 抽出した行全体についての性質をcolsummaryで調べる
$ colgrep -c11 -e'^$' T*0710.txt | colsummary
```

### 結果

```
match:134 unmatch:1360 total:1494 (colgrep)
cpos	diff	ave.	range	frequent	frequency(multi)	digits
1	134	978.000	978-4-06-522875-3~978-4-86468-817-8	978-4-06-523796-0|978-4-10-772399-4|978-4-86468-817-8|978-4-7964-1454-8|978-4-7997-5333-0|978-4-396-76825-6	1(134)	17
2	131	0.381	18歳、新妻、不倫します。 5~黒龍に蝶のキス	THE IDOLM@STER MILLION LIVE!THEATER DAYS Brand New Song 4|僕の心のヤバイやつ 5|お稲荷JKたまもちゃん! 6|黒龍に蝶のキス|仮面ライダークウガ 17|冰剣の魔術師が世界を統べる 4	2(3)|1(128)	2~57
3	21	0.000	|#たっぷり濡らす#その後は…~魔物の森で最強の大魔帝国を作り上げる	|俺の異世界転生冒険譚|Fate/Grand Order-Epic of Remnant-亜種特異点3/亜種並行世界屍山血河舞台下総国|世界最強の魔術師である少年は、魔術学院に入学する|迷宮と高校生|奈落で鍛えた最強の<神眼>で無双する	114|1(20)	0|6~55
4	126	0.000	KAKERU~龍牙翔 漫画	田中光|車田正美 著|斗田めぐみ 著|ima 漫画|桜井のりお|ユウキレイ 著	3(2)|2(4)|1(120)	3~12
5	40	0.000	|TYPE-MOON 原作~高橋ヒロシ 原作	|バンダイナムコエンターテインメント 原作|進行諸島 原作|木嶋隆太 原作|伏瀬 原作|ヨコオタロウ 原作・監修90|3|2(3)|1(35)	0|5~20
6	3	0.000	|CD付き特装版~特装版	|特装版|CD付き特装版	131|2|10|3~7
7	18	0.000	スクウェア・エニックス~講談社	講談社|秋田書店|スクウェア・エニックス|白泉社|ハーパーコリンズ・ジャパン|祥伝社	42|14(2)|7(3)|6(3)|5(3)|4|2|1(4)	3~13
8	3	0.000	|小学館クリエイティブ~講談社	|講談社|小学館クリエイティブ	128|5|1	0|3~10
9	1	2021.700	2021.7	2021.7	134	6
10	1	0.000			134	0
11	1	0.000			134	0
12	2	0.000	|録音ディスク(1枚 12cm)	|録音ディスク(1枚 12cm)	133|1	0|15
13	34	0.000	BAMBOO COMICS~週刊少年マガジン	週刊少年マガジン|ガンガンコミックスUP!|少年チャンピオン・コミックス|月刊少年シリウス|ヤングマガジン|ハーレクインコミックス★キララ	21|14|10|9|8|7|6(2)|5|4(3)|3(3)|2(8)|1(11)	2~28
14	10	0.000	|<2 in 1>コミックス~麗人uno!	|GUSH COMICS|LaLa|<2 in 1>コミックス|億万長者に愛されて 3|麗人uno!	117|6|4|1(7)	0|4~13
15	1	0.000			134	0
16	1	0.000			134	0
17	33	0.000	¥1000~¥940	¥650|¥600|¥680|¥454|¥480|¥660	20|19|11|9|8|7|6|4(5)|3(3)|2(7)|1(11)	4~5
18	1	0.000			134	0
134 line(s) read; 0.008468 seconds (colsummary)
※11列目が空なら10、15、16、18列目もすべて空とわかる
```

## colgrepの結果を比較する(2)
```
# 比較のために、上記で抽出した行以外（-~指定）を調べる
$ colgrep -~ -c11 -e'^$' T*0710.txt | colsummary
```

### 結果

```
match:1360 unmatch:134 total:1494 (colgrep)
cpos	diff	ave.	range	frequent	frequency(multi)	digits
1	1351	972.966	|978-4-01-034649-5~978-4-9912014-0-0	|978-4-324-10918-2|978-4-434-29038-1|978-4-521-74903-7|978-4-7999-0428-2|978-4-7581-2114-9	7|3|2|1(1348)	0|17
2	1358	  NaN	#や【フラット】が出てこないから、1本で楽しめる複音ハーモニカの本昭和のうた~龍馬と西郷	一夜の夢|THE STONE|運慶と快慶。|神戸市の一般教養参考書 '23年度版|百花|ミャンマー政変	2(2)|1(1356)	1~74
3	547	1.918	|130問~黒糸1色で描く美しい幾何学模様 詳しい刺し方付きパターン集	|大学入試|国立私立|高校入試|日本旅行文化協会|モダン語辞典・事典・用語編	775|8|5(2)|4(2)|3(3)|2(12)|1(526)	0|2~72
4	1003	0.000	|AYA 著~龍子 著	|協同教育研究会 編|日本医歯薬研修協会臨床検査技師国家試験対策課国家試験問題解説書編集委員会 編著|全国入試模試センター 編|荒井賢一 著|きんざいファイナンシャル・プランナーズ・センター 編著	231|41|8|7|6(3)|4(2)|3(6)|2(41)|1(947)	0|3~39
5	314	0.000	|DIAMONDハーバード・ビジネス・レビュー編集部 訳~黒輪篤嗣 訳	|荒山正彦 監修|須田将司 編集・解説|笠置隆範 編著|吉岡剛 原作|三条陸 原作	1026|4|3(5)|2(8)|1(299)	0|3~34
6	34	1.501	|2022年度入試用限定版~第6版	|改訂版|復刻|第2版|改訂2版|新装版	1252|23|14|8|7|6|5(2)|3(4)|2(6)|1(16)	0|2~12
7	493	0.000	|ASK~黎明書房	KADOKAWA|協同出版|教英出版|小学館|講談社|学研プラス	70|41|32|31|24|18~6(13)|5(17)|4(30)|3(37)|2(68)|1(292)	0|2~31
8	69	0.000	|JRC~風媒社	|星雲社|幻冬舎|つちや書店|日経BPマーケティング|中央経済グループパブリッシング	1212|28|12|8|7|5|4(4)|3(3)|2(7)|1(49)	0|3~15
9	8	2017.211	2021~2021.8|c2021	2021.7|2021.6|2021.8|2021|2021.5|2021.3	1034|187|79|33|16|5|3(2)	4~6
10	443	212.996	10, 169p~p942〜1064	1冊(ページ付なし)|1冊|159p|127p|95p|191p	52|43|30|23|17|16(2)~6(19)|5(28)|4(42)|3(60)|2(77)|1(186)	2~14
11	40	21.800	13×13cm~9.7×12cm	21cm|19cm|26cm|15cm|30cm|18cm	281|260|254|85|78|75~10|9(2)|8(2)|4|2(7)|1(14)	4~8
12	7	0.000	|CD-ROM(1枚 12cm)~録音ディスク(3枚 12cm)	|録音ディスク(1枚 12cm)|CD-ROM(1枚 12cm)|録音ディスク(2枚 12cm)|DVD-ROM(1枚 12cm)|録音ディスク(3枚 12cm)	1340|11|4|2|1(3)0|15~19
13	498	0.008	|&arche NOVELS~鹿児島県の教員採用試験参考書シリーズ 2	|駿台大学入試完全対策シリーズ|角川コミックス・エース|ドラゴンコミックスエイジ|愛蔵版コミックス|るるぶ情報版	753|13|10|8|6(2)|5(2)|4(7)|3(9)|2(25)|1(449)	0|2~34
14	40	0.000	|ALBA TROSS-VIEW~風烈廻り与力・青柳剣一郎 54	|光文社時代小説文庫|レジーナブックス|エタニティブックス|テレビマガジン|小学館時代小説文庫	1308|5|3(2)|2(5)|1(31)	0|4~33
15	7	0.000	|Rouge~隠密船頭 7	|Rouge|車種別チューニング&ドレスアップ徹底ガイドシリーズ vol.254|隠密船頭 7|吟味方与力望月城之進 3|ゴルフ上達のトリセツ	1352|3|1(5)	0|5~33
16	165	7.113	|1925(大正14)年9月〜12月~饗宴	|口腔外科学 1|最終決戦 3|特集・ソロ活ビジネス最前線!|特集「女性のサルコペニアを考える」|病理組織細胞学	1196|1(164)	0|2~67
17	197	0.000	¥1000~¥9900	¥1600|¥1500|¥1800|¥1300|¥1200|¥2000	96|71|69|66|56|51~6(5)|5(12)|4(11)|3(20)|2(23)|1(87)	4~6
18	5	0.021	|2巻セット¥2700~全3巻セット¥23000	|4巻セット¥120000|3巻セット¥62000|全3巻セット¥23000|2巻セット¥2700	1348|4|3(2)|2	0|10~12
1,360 line(s) read; 0.081094 seconds (colsummary)
※比較してわかることは、すべてが空でないのは10列目のみ
```
